package guillermocs.patrimonio30

import android.os.Bundle
import android.support.v4.app.Fragment
import android.text.util.Linkify
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.google.android.gms.maps.*
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.model.Marker
import com.google.android.gms.maps.model.MarkerOptions


/**
 * @author Guillermo Cebrián Serrano
 * Fragment que se encarga de mostrar una informacion para viajar al usuario
 * se usa el metodo newInstance para crear una nueva instancia
 */
class TravelFragment : Fragment(), OnMapReadyCallback {

    private lateinit var location: String
    private lateinit var urls: String

    private lateinit var tv_travel_urls: TextView
    private lateinit var googleMap: GoogleMap

    companion object {
        private const val TAG = "TravelFragment"

        private const val TRAVEL_LOCATION = "Location"
        private const val TRAVEL_URLS = "Urls"
        /**
         * Crea una nueva instancia de TravelFragment con los parametros pasados
         *
         * @param location Ubicación del sitio
         * @param urls Urls de interes del sitio
         * @return Una nueva instancia de TravelFragment.
         */

        @JvmStatic
        fun newInstance(location: String, urls: String) =
                TravelFragment().apply {
                    arguments = Bundle().apply {
                        putString(TRAVEL_LOCATION, location)
                        putString(TRAVEL_URLS, urls)
                    }
                }
    }

    /**
     * Metodo que se encarga de instanciar un nuevo fragment de tipo TravelFrgament
     * @param savedInstanceState que contiene la localizacion y las urls de interes de un sitio
     */
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            location = it.getString(TRAVEL_LOCATION)
            urls = it.getString(TRAVEL_URLS)
        }
    }


    /**
     * Metodo que se encarga de crear la vista
     * @return Una nueva vista de tipo TravelFragment
     */
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {

        // Rellenamos el layout de este fragmento
        val v = inflater.inflate(R.layout.fragment_travel, container, false)

        // Asignamos las variables de la vista a las variables del fragment
        tv_travel_urls = v.findViewById(R.id.travel_text_urls)

        //Obtiene el fragmento que soporta el mapa y es notificado cuando el mapa esta listo para ser usado
        var mapFragment = SupportMapFragment.newInstance(GoogleMapOptions().mapType(GoogleMap.MAP_TYPE_TERRAIN))
        fragmentManager!!.beginTransaction().replace(R.id.travel_fragment_map, mapFragment).commit()
        mapFragment.getMapAsync(this)

        // Asignamos los valores a las vistas
        tv_travel_urls.text = urls
        // Miramos si hay algún url dentro del texto
        Linkify.addLinks(tv_travel_urls, Linkify.WEB_URLS)


        return v
    }

    /**
     * @param map Mapa que se recibe cunado esta listo para usarse
     */
    override fun onMapReady(map: GoogleMap) {

        // Se inicializa el mapa del fragment con el mapa recibido
        googleMap = map

        // Obtenemos las coordenadas
        val auxloc = stringtolatlng(this.location)
        //Log.d(TAG, String.format("Localizacion:%s \tAuxLoc:%s ", this.location, auxloc.toString()))

        // Añadimos un marcador en la ubicación y movemos el focus del mapa
        val markerOptions = MarkerOptions().position(auxloc)
        googleMap.addMarker(markerOptions)
        googleMap.moveCamera(CameraUpdateFactory.newLatLngZoom(auxloc, 7.0f))
    }

    /**
     * Metodo que devuelve la ubicación transformada
     * @param location ubicacion en formato string latitud y longitud separadas por comas
     * @return La ubicacion en formato LatLong
     */
    private fun stringtolatlng(location: String): LatLng {

        val latlong = location.split(",".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
        val latitude = java.lang.Double.parseDouble(latlong[0])
        val longitude = java.lang.Double.parseDouble(latlong[1])
        return LatLng(latitude, longitude)
    }


}

